"""FHIR R4 resource transformers.

Pure functions converting Django model instances to FHIR R4 JSON dicts.
No fhir.resources validation at serve time for performance.
"""

import base64
from datetime import date


def _isoformat(dt):
    """Format a datetime/date to ISO string, or None."""
    if dt is None:
        return None
    return dt.isoformat() if hasattr(dt, 'isoformat') else str(dt)


def patient_to_fhir(patient) -> dict:
    """Convert Patient model to FHIR Patient resource."""
    birth_year = patient.anchor_year - patient.anchor_age
    resource = {
        'resourceType': 'Patient',
        'id': f'mimic-{patient.subject_id}',
        'identifier': [{
            'system': 'http://mimic.mit.edu/fhir/identifier/patient',
            'value': str(patient.subject_id),
        }],
        'gender': 'male' if patient.gender == 'M' else 'female',
        'birthDate': f'{birth_year}-01-01',
    }
    if patient.dod:
        resource['deceasedDateTime'] = _isoformat(patient.dod)
    return resource


def encounter_to_fhir(encounter) -> dict:
    """Convert Encounter model to FHIR Encounter resource."""
    admission_type = encounter.admission_type.upper() if encounter.admission_type else ''
    if 'EMERGENCY' in admission_type or 'URGENT' in admission_type:
        act_code = 'EMER'
    elif 'ELECTIVE' in admission_type or 'SURGICAL' in admission_type:
        act_code = 'AMB'
    else:
        act_code = 'IMP'

    resource = {
        'resourceType': 'Encounter',
        'id': f'mimic-{encounter.hadm_id}',
        'status': 'finished',
        'class': {
            'system': 'http://terminology.hl7.org/CodeSystem/v3-ActCode',
            'code': act_code,
        },
        'subject': {'reference': f'Patient/mimic-{encounter.patient.subject_id}'},
        'period': {
            'start': _isoformat(encounter.admittime),
        },
    }
    if encounter.dischtime:
        resource['period']['end'] = _isoformat(encounter.dischtime)
    hospitalization = {}
    if encounter.admission_location:
        hospitalization['admitSource'] = {'text': encounter.admission_location}
    if encounter.discharge_location:
        hospitalization['dischargeDisposition'] = {'text': encounter.discharge_location}
    if hospitalization:
        resource['hospitalization'] = hospitalization
    return resource


def lab_to_fhir(lab) -> dict:
    """Convert LabEvent model to FHIR Observation (laboratory)."""
    resource = {
        'resourceType': 'Observation',
        'id': f'mimic-lab-{lab.labevent_id}',
        'status': 'final',
        'category': [{
            'coding': [{
                'system': 'http://terminology.hl7.org/CodeSystem/observation-category',
                'code': 'laboratory',
                'display': 'Laboratory',
            }],
        }],
        'code': {'text': lab.label},
        'subject': {'reference': f'Patient/mimic-{lab.patient.subject_id}'},
        'effectiveDateTime': _isoformat(lab.charttime),
    }
    if lab.encounter_id:
        resource['encounter'] = {'reference': f'Encounter/mimic-{lab.encounter.hadm_id}'}
    if lab.valuenum is not None:
        resource['valueQuantity'] = {
            'value': lab.valuenum,
            'unit': lab.valueuom or '',
        }
    elif lab.value:
        resource['valueString'] = lab.value
    ref_range = {}
    if lab.ref_range_lower is not None:
        ref_range['low'] = {'value': lab.ref_range_lower}
    if lab.ref_range_upper is not None:
        ref_range['high'] = {'value': lab.ref_range_upper}
    if ref_range:
        resource['referenceRange'] = [ref_range]
    if lab.flag:
        resource['interpretation'] = [{'text': lab.flag}]
    return resource


def vital_to_fhir(vital) -> dict:
    """Convert VitalSign model to FHIR Observation (vital-signs)."""
    resource = {
        'resourceType': 'Observation',
        'id': f'mimic-vital-{vital.pk}',
        'status': 'final',
        'category': [{
            'coding': [{
                'system': 'http://terminology.hl7.org/CodeSystem/observation-category',
                'code': 'vital-signs',
                'display': 'Vital Signs',
            }],
        }],
        'code': {'text': vital.label},
        'subject': {'reference': f'Patient/mimic-{vital.patient.subject_id}'},
        'effectiveDateTime': _isoformat(vital.charttime),
    }
    if vital.encounter_id:
        resource['encounter'] = {'reference': f'Encounter/mimic-{vital.encounter.hadm_id}'}
    if vital.valuenum is not None:
        resource['valueQuantity'] = {
            'value': vital.valuenum,
            'unit': vital.valueuom or '',
        }
    elif vital.value:
        resource['valueString'] = vital.value
    return resource


def diagnosis_to_fhir(dx) -> dict:
    """Convert Diagnosis model to FHIR Condition resource."""
    if dx.icd_version == 10:
        system = 'http://hl7.org/fhir/sid/icd-10-cm'
    else:
        system = 'http://hl7.org/fhir/sid/icd-9-cm'

    return {
        'resourceType': 'Condition',
        'id': f'mimic-dx-{dx.encounter.hadm_id}-{dx.seq_num}',
        'subject': {'reference': f'Patient/mimic-{dx.patient.subject_id}'},
        'encounter': {'reference': f'Encounter/mimic-{dx.encounter.hadm_id}'},
        'code': {
            'coding': [{
                'system': system,
                'code': dx.icd_code,
                'display': dx.long_title,
            }],
            'text': dx.long_title,
        },
    }


def procedure_to_fhir(proc) -> dict:
    """Convert Procedure model to FHIR Procedure resource."""
    if proc.icd_version == 10:
        system = 'http://hl7.org/fhir/sid/icd-10-pcs'
    else:
        system = 'http://hl7.org/fhir/sid/icd-9-cm'

    return {
        'resourceType': 'Procedure',
        'id': f'mimic-proc-{proc.encounter.hadm_id}-{proc.seq_num}',
        'status': 'completed',
        'subject': {'reference': f'Patient/mimic-{proc.patient.subject_id}'},
        'encounter': {'reference': f'Encounter/mimic-{proc.encounter.hadm_id}'},
        'code': {
            'coding': [{
                'system': system,
                'code': proc.icd_code,
                'display': proc.long_title,
            }],
            'text': proc.long_title,
        },
    }


def medication_to_fhir(med) -> dict:
    """Convert Medication model to FHIR MedicationRequest resource."""
    resource = {
        'resourceType': 'MedicationRequest',
        'id': f'mimic-med-{med.pk}',
        'status': 'completed',
        'intent': 'order',
        'medicationCodeableConcept': {
            'text': med.drug,
        },
        'subject': {'reference': f'Patient/mimic-{med.patient.subject_id}'},
        'encounter': {'reference': f'Encounter/mimic-{med.encounter.hadm_id}'},
    }
    if med.ndc:
        resource['medicationCodeableConcept']['coding'] = [{
            'system': 'http://www.nlm.nih.gov/research/umls/rxnorm',
            'code': med.ndc,
        }]
    dosage = {}
    if med.dose_val_rx or med.dose_unit_rx:
        dose_quantity = {}
        if med.dose_val_rx:
            try:
                dose_quantity['value'] = float(med.dose_val_rx)
            except (ValueError, TypeError):
                dose_quantity['value'] = med.dose_val_rx
        if med.dose_unit_rx:
            dose_quantity['unit'] = med.dose_unit_rx
        if dose_quantity:
            dosage['doseAndRate'] = [{'doseQuantity': dose_quantity}]
    if med.route:
        dosage['route'] = {'text': med.route}
    if dosage:
        resource['dosageInstruction'] = [dosage]
    return resource


def note_to_fhir(note) -> dict:
    """Convert Note model to FHIR DocumentReference with base64 content."""
    encoded_text = base64.b64encode(note.text.encode('utf-8')).decode('ascii')
    resource = {
        'resourceType': 'DocumentReference',
        'id': f'mimic-note-{note.note_id}',
        'status': 'current',
        'type': {'text': note.note_type},
        'subject': {'reference': f'Patient/mimic-{note.patient.subject_id}'},
        'content': [{
            'attachment': {
                'contentType': 'text/plain',
                'data': encoded_text,
            },
        }],
    }
    if note.encounter_id:
        resource['context'] = {
            'encounter': [{'reference': f'Encounter/mimic-{note.encounter.hadm_id}'}],
        }
    if note.charttime:
        resource['date'] = _isoformat(note.charttime)
    return resource


def make_bundle(entries, bundle_type='searchset', total=None, links=None) -> dict:
    """Wrap a list of FHIR resources into a Bundle."""
    bundle = {
        'resourceType': 'Bundle',
        'type': bundle_type,
        'entry': [{'resource': e, 'fullUrl': f"{e['resourceType']}/{e['id']}"} for e in entries],
    }
    if total is not None:
        bundle['total'] = total
    if links:
        bundle['link'] = links
    return bundle



# Generated by Django 6.0.2 on 2026-03-02 05:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CohortDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(blank=True)),
                ('criteria', models.JSONField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('patient_count', models.IntegerField(null=True)),
                ('encounter_count', models.IntegerField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name='SavedQuery',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(blank=True)),
                ('query_definition', models.JSONField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('result_count', models.IntegerField(null=True)),
            ],
            options={
                'verbose_name_plural': 'Saved queries',
            },
        ),
        migrations.CreateModel(
            name='CohortMember',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('patient_id', models.IntegerField()),
                ('encounter_id', models.IntegerField(blank=True, null=True)),
                ('group_label', models.CharField(blank=True, max_length=50)),
                ('cohort', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='members', to='research.cohortdefinition')),
            ],
            options={
                'unique_together': {('cohort', 'patient_id', 'encounter_id')},
            },
        ),
    ]



from django.db import models

# Clinical models (LabEvent, VitalSign, Diagnosis, Procedure, Medication,
# MedicationAdministration, Note) have been migrated to DuckDB.
# See core/duckdb_schema.py for table definitions.

VITAL_SIGN_ITEMIDS = {
    220045: 'Heart Rate',
    220050: 'Arterial Blood Pressure systolic',
    220051: 'Arterial Blood Pressure diastolic',
    220052: 'Arterial Blood Pressure mean',
    220179: 'Non Invasive Blood Pressure systolic',
    220180: 'Non Invasive Blood Pressure diastolic',
    220181: 'Non Invasive Blood Pressure mean',
    220210: 'Respiratory Rate',
    220277: 'SpO2',
    223761: 'Temperature Fahrenheit',
    223762: 'Temperature Celsius',
    226512: 'Weight (kg)',
    226730: 'Height (cm)',
}



"""Management command for importing MIMIC-IV data."""
from django.core.management.base import BaseCommand, CommandError
from core.importer import MIMICImporter, validate_mimic_folder


class Command(BaseCommand):
    help = 'Import MIMIC-IV data from CSV files into the database'

    def add_arguments(self, parser):
        parser.add_argument('folder_path', type=str, help='Path to the MIMIC-IV data folder')
        parser.add_argument(
            '--modules', nargs='+', default=['hosp'],
            choices=['hosp', 'icu', 'note'],
            help='Modules to import (default: hosp)',
        )
        parser.add_argument(
            '--patient-limit', type=int, default=None,
            help='Limit number of patients to import',
        )
        parser.add_argument(
            '--generate-fhir', action='store_true',
            help='Generate FHIR resources during import',
        )

    def handle(self, *args, **options):
        folder_path = options['folder_path']

        # Validate folder
        self.stdout.write(f'Validating folder: {folder_path}')
        result = validate_mimic_folder(folder_path)
        if not result['valid']:
            raise CommandError(
                f"Invalid MIMIC folder. Missing required files: {result['missing_required']}"
            )
        self.stdout.write(self.style.SUCCESS('Folder validation passed'))

        # Run import
        importer = MIMICImporter(
            folder_path=folder_path,
            modules=options['modules'],
            patient_limit=options['patient_limit'],
            generate_fhir=options['generate_fhir'],
        )

        self.stdout.write('Starting import...')
        importer.run()
        self.stdout.write(self.style.SUCCESS('Import completed successfully!'))



import os
import django

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'mimic_explorer.settings')
django.setup()

from pathlib import Path
from django.test import TestCase
from rest_framework.test import APIClient
from core.models import SystemConfig
from core.importer import MIMICImporter
from core.duckdb_manager import get_connection
from research.models import CohortMember, CohortDefinition, SavedQuery
from tests.duckdb_test_utils import DuckDBTestMixin

FIXTURE_PATH = Path(__file__).parent / 'fixtures' / 'mimic-iv-test'


class StatusAPITest(TestCase):
    def setUp(self):
        self.client = APIClient()

    def test_status_endpoint(self):
        """GET /api/status/ should return system configuration."""
        response = self.client.get('/api/status/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/status/ failed. This is the main status endpoint. Check that "
            "core/urls.py has a path for 'status/' and that StatusView in "
            "core/views.py returns the SystemConfig data."
        )
        self.assertIn(
            'import_status', response.json(),
            "Status response is missing 'import_status' field. Check the "
            "StatusView in core/views.py includes all fields."
        )
        self.assertEqual(
            response.json()['import_status'], 'not_started',
            "Fresh install should show import_status='not_started'. Check "
            "SystemConfig default values in core/models.py."
        )


class ValidateFolderAPITest(TestCase):
    def setUp(self):
        self.client = APIClient()

    def test_validate_valid_folder(self):
        """POST /api/import/validate-folder/ should accept a valid MIMIC folder."""
        response = self.client.post(
            '/api/import/validate-folder/',
            {'folder_path': str(FIXTURE_PATH)},
            format='json',
        )
        self.assertEqual(
            response.status_code, 200,
            f"Folder validation returned {response.status_code}. Check that "
            "ValidateFolderView in core/views.py accepts POST with folder_path."
        )
        self.assertTrue(
            response.json()['valid'],
            f"Fixture folder was not detected as valid. Path: {FIXTURE_PATH}. "
            "Check that the fixture folder contains the required hosp/ CSV files."
        )

    def test_validate_invalid_folder(self):
        """POST /api/import/validate-folder/ should reject a nonexistent path."""
        response = self.client.post(
            '/api/import/validate-folder/',
            {'folder_path': '/nonexistent'},
            format='json',
        )
        self.assertEqual(
            response.status_code, 200,
            "Folder validation should return 200 even for invalid folders (with "
            "valid=false in the body), not an HTTP error."
        )
        self.assertFalse(
            response.json()['valid'],
            "Nonexistent folder should return valid=false."
        )


class PatientAPITest(DuckDBTestMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._setup_duckdb()
        importer = MIMICImporter(
            folder_path=str(FIXTURE_PATH),
            modules=['hosp'],
        )
        importer.run()

    @classmethod
    def tearDownClass(cls):
        cls._teardown_duckdb()
        super().tearDownClass()

    def setUp(self):
        self.client = APIClient()

    def test_patient_list(self):
        """GET /api/patients/ should return paginated patient list."""
        response = self.client.get('/api/patients/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/patients/ failed. Check patients/urls.py and patients/views.py "
            "for the patient list endpoint."
        )
        data = response.json()
        self.assertIn(
            'results', data,
            "Patient list response should be paginated with a 'results' key. "
            "Check that DRF pagination is configured in settings.py."
        )
        self.assertEqual(
            len(data['results']), 10,
            f"Expected 10 patients from fixtures, got {len(data['results'])}. "
            "The import may have failed or the fixture data may be incomplete."
        )

    def test_patient_filter_gender(self):
        """GET /api/patients/?gender=M should only return male patients."""
        response = self.client.get('/api/patients/', {'gender': 'M'})
        self.assertEqual(response.status_code, 200)
        for p in response.json()['results']:
            self.assertEqual(
                p['gender'], 'M',
                f"Gender filter returned a patient with gender '{p['gender']}'. "
                "Check the gender filter in the Patient list view."
            )

    def test_patient_detail(self):
        """GET /api/patients/:subject_id/ should return patient with encounters."""
        response = self.client.get('/api/patients/10001/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/patients/10001/ failed. Check the patient detail view uses "
            "subject_id (not pk) for lookup. The URL pattern should use "
            "<int:subject_id>."
        )
        data = response.json()
        self.assertEqual(
            data['subject_id'], 10001,
            "Patient detail returned wrong subject_id."
        )
        self.assertIn(
            'encounters', data,
            "Patient detail should include nested 'encounters'. Check the "
            "PatientDetailView includes encounters."
        )

    def test_patient_not_found(self):
        """GET /api/patients/99999/ should return 404 for unknown patient."""
        response = self.client.get('/api/patients/99999/')
        self.assertEqual(
            response.status_code, 404,
            "Non-existent patient should return 404. Check the patient detail "
            "view handles missing patients."
        )

    def test_encounter_list(self):
        """GET /api/encounters/ should return paginated encounter list."""
        response = self.client.get('/api/encounters/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/encounters/ failed. Check patients/urls.py for the "
            "encounters list endpoint."
        )
        self.assertIn(
            'results', response.json(),
            "Encounter list should be paginated with a 'results' key."
        )

    def test_encounter_detail(self):
        """GET /api/encounters/:hadm_id/ should return encounter details."""
        response = self.client.get('/api/encounters/20001/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/encounters/20001/ failed. Check the encounter detail view "
            "uses hadm_id for lookup."
        )
        self.assertEqual(
            response.json()['hadm_id'], 20001,
            "Encounter detail returned wrong hadm_id."
        )

    def test_encounter_labs(self):
        """GET /api/encounters/:hadm_id/labs/ should return lab events."""
        response = self.client.get('/api/encounters/20001/labs/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/encounters/20001/labs/ failed. Check that the encounter "
            "sub-resource URL for labs is registered in patients/urls.py."
        )

    def test_encounter_diagnoses(self):
        """GET /api/encounters/:hadm_id/diagnoses/ should return diagnoses."""
        response = self.client.get('/api/encounters/20001/diagnoses/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/encounters/20001/diagnoses/ failed. Check the encounter "
            "diagnoses sub-resource endpoint."
        )

    def test_patient_timeline(self):
        """GET /api/patients/:subject_id/timeline/ should return event list."""
        response = self.client.get('/api/patients/10001/timeline/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/patients/10001/timeline/ failed. Check the timeline view "
            "in patients/views.py."
        )
        self.assertIsInstance(
            response.json(), list,
            "Timeline endpoint should return a JSON array of events, not an object. "
            "Check the timeline view's Response format."
        )


class ClinicalAPITest(DuckDBTestMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._setup_duckdb()
        importer = MIMICImporter(
            folder_path=str(FIXTURE_PATH),
            modules=['hosp'],
        )
        importer.run()

    @classmethod
    def tearDownClass(cls):
        cls._teardown_duckdb()
        super().tearDownClass()

    def setUp(self):
        self.client = APIClient()

    def test_labs_list(self):
        """GET /api/labs/ should return paginated lab events."""
        response = self.client.get('/api/labs/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/labs/ failed. Check clinical/urls.py for the labs list "
            "endpoint and clinical/views.py for the view."
        )
        self.assertIn(
            'results', response.json(),
            "Labs list should be paginated with a 'results' key."
        )

    def test_labs_filter_patient(self):
        """GET /api/labs/?patient=10001 should filter by subject_id."""
        response = self.client.get('/api/labs/', {'patient': '10001'})
        self.assertEqual(
            response.status_code, 200,
            "GET /api/labs/?patient=10001 failed. Check that the labs view "
            "supports filtering by 'patient' (subject_id) parameter."
        )

    def test_diagnoses_list(self):
        """GET /api/diagnoses/ should return paginated diagnoses."""
        response = self.client.get('/api/diagnoses/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/diagnoses/ failed. Check clinical/urls.py for the diagnoses "
            "list endpoint."
        )

    def test_medications_list(self):
        """GET /api/medications/ should return paginated medications."""
        response = self.client.get('/api/medications/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/medications/ failed. Check clinical/urls.py for the "
            "medications list endpoint."
        )

    def test_lab_items(self):
        """GET /api/lab-items/ should return distinct lab item IDs."""
        response = self.client.get('/api/lab-items/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/lab-items/ failed. This endpoint should return a list of "
            "distinct {itemid, label} objects from imported lab events."
        )

    def test_vital_items(self):
        """GET /api/vital-items/ should return the list of tracked vital sign types."""
        response = self.client.get('/api/vital-items/')
        self.assertEqual(
            response.status_code, 200,
            "GET /api/vital-items/ failed. This endpoint returns the VITAL_SIGN_ITEMIDS "
            "mapping from clinical/models.py."
        )
        items = response.json()
        self.assertGreater(
            len(items), 0,
            "Vital items list is empty. This endpoint should return the "
            "VITAL_SIGN_ITEMIDS dict (13 vital sign types like Heart Rate, SpO2, etc.) "
            "regardless of whether any vitals have been imported."
        )


class ResetAPITest(DuckDBTestMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._setup_duckdb()
        importer = MIMICImporter(
            folder_path=str(FIXTURE_PATH),
            modules=['hosp'],
        )
        importer.run()

    @classmethod
    def tearDownClass(cls):
        cls._teardown_duckdb()
        super().tearDownClass()

    def setUp(self):
        self.client = APIClient()

    def test_reset_clears_all_data(self):
        """POST /api/reset/ should wipe all tables and reset config."""
        conn = get_connection()
        # Verify data exists before reset
        self.assertGreater(
            conn.execute("SELECT COUNT(*) FROM patient").fetchone()[0], 0,
            "No patients in DB before reset test — the import may have failed."
        )
        self.assertGreater(
            conn.execute("SELECT COUNT(*) FROM encounter").fetchone()[0], 0,
            "No encounters in DB before reset test — the import may have failed."
        )
        config = SystemConfig.get_solo()
        self.assertEqual(config.import_status, 'completed')

        # Perform reset
        response = self.client.post('/api/reset/')
        self.assertEqual(
            response.status_code, 200,
            f"POST /api/reset/ returned {response.status_code}. Check ResetView "
            "in core/views.py."
        )
        self.assertEqual(
            response.json()['status'], 'reset_complete',
            "Reset response should return {status: 'reset_complete'}."
        )

        # Verify all DuckDB tables are empty
        for table, name in [
            ('patient', 'patients'), ('encounter', 'encounters'),
            ('icu_stay', 'ICU stays'), ('lab_event', 'lab events'),
            ('vital_sign', 'vital signs'), ('diagnosis', 'diagnoses'),
            ('procedure', 'procedures'), ('medication', 'medications'),
            ('note', 'notes'),
        ]:
            count = conn.execute(f"SELECT COUNT(*) FROM {table}").fetchone()[0]
            self.assertEqual(
                count, 0,
                f"Reset did not clear {name} table ({count} rows remain). "
                f"Check that ResetView calls reset_all_tables() from duckdb_queries."
            )

        # Verify SQLite app-state tables are cleared
        self.assertEqual(
            CohortDefinition.objects.count(), 0,
            "Reset did not clear cohort definitions."
        )
        self.assertEqual(
            SavedQuery.objects.count(), 0,
            "Reset did not clear saved queries."
        )

        # Verify SystemConfig is reset
        config = SystemConfig.get_solo()
        self.assertEqual(
            config.import_status, 'not_started',
            "Reset should set import_status back to 'not_started'."
        )
        self.assertEqual(
            config.total_patients, 0,
            "Reset should set total_patients to 0."
        )
        self.assertEqual(
            config.total_encounters, 0,
            "Reset should set total_encounters to 0."
        )
        self.assertEqual(
            config.mimic_data_path, '',
            "Reset should clear mimic_data_path."
        )
        self.assertEqual(
            config.import_progress, {},
            "Reset should clear import_progress to {}."
        )
        self.assertEqual(
            config.imported_modules, [],
            "Reset should clear imported_modules to []."
        )


class SupplementImportAPITest(DuckDBTestMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._setup_duckdb()
        importer = MIMICImporter(
            folder_path=str(FIXTURE_PATH),
            modules=['hosp'],
        )
        importer.run()

    @classmethod
    def tearDownClass(cls):
        cls._teardown_duckdb()
        super().tearDownClass()

    def setUp(self):
        self.client = APIClient()

    def test_supplement_import_requires_completed(self):
        """POST /api/import/supplement/ should return 400 if initial import not done."""
        config = SystemConfig.get_solo()
        config.import_status = 'not_started'
        config.save()

        response = self.client.post(
            '/api/import/supplement/',
            {
                'folder_path': str(FIXTURE_PATH),
                'modules': ['hosp', 'icu'],
                'patient_limit': 100,
                'generate_fhir': False,
            },
            format='json',
        )
        self.assertEqual(
            response.status_code, 400,
            "Supplement import should return 400 when import_status is not "
            "'completed'. You must complete an initial import before supplementing. "
            "Check SupplementImportView in core/views.py."
        )

        # Restore
        config.import_status = 'completed'
        config.save()

    def test_supplement_import_no_duplicates(self):
        """Re-importing the same data should not create duplicate rows."""
        conn = get_connection()
        patient_count_before = conn.execute("SELECT COUNT(*) FROM patient").fetchone()[0]
        encounter_count_before = conn.execute("SELECT COUNT(*) FROM encounter").fetchone()[0]
        diagnosis_count_before = conn.execute("SELECT COUNT(*) FROM diagnosis").fetchone()[0]

        config = SystemConfig.get_solo()
        self.assertEqual(config.import_status, 'completed')
        self.assertEqual(
            config.imported_modules, ['hosp'],
            "Expected only 'hosp' in imported_modules before supplement test."
        )

        # Run supplement import synchronously (not via API thread)
        importer = MIMICImporter(
            folder_path=str(FIXTURE_PATH),
            modules=['hosp', 'icu'],
        )
        importer.run()

        # Counts should not increase for hosp-only data
        patient_count_after = conn.execute("SELECT COUNT(*) FROM patient").fetchone()[0]
        encounter_count_after = conn.execute("SELECT COUNT(*) FROM encounter").fetchone()[0]
        diagnosis_count_after = conn.execute("SELECT COUNT(*) FROM diagnosis").fetchone()[0]

        self.assertEqual(
            patient_count_after, patient_count_before,
            f"Patient count changed from {patient_count_before} to "
            f"{patient_count_after} after supplement import. Re-importing "
            "should be idempotent — check ON CONFLICT DO NOTHING on patient table."
        )
        self.assertEqual(
            encounter_count_after, encounter_count_before,
            f"Encounter count changed from {encounter_count_before} to "
            f"{encounter_count_after} after supplement import. Re-importing "
            "should be idempotent — check ON CONFLICT DO NOTHING on encounter table."
        )
        self.assertEqual(
            diagnosis_count_after, diagnosis_count_before,
            f"Diagnosis count changed from {diagnosis_count_before} to "
            f"{diagnosis_count_after} after supplement import. Duplicate "
            "diagnoses were created. Check UNIQUE constraint on diagnosis table."
        )

        # imported_modules should now include both
        config.refresh_from_db()
        self.assertIn(
            'hosp', config.imported_modules,
            "imported_modules should still contain 'hosp' after supplement."
        )
        self.assertIn(
            'icu', config.imported_modules,
            "imported_modules should contain 'icu' after supplementing with ICU data. "
            "Check that _finalize() in core/importer.py merges modules with existing."
        )

    def test_supplement_endpoint_starts(self):
        """POST /api/import/supplement/ should return 202 and complete in background."""
        config = SystemConfig.get_solo()
        config.import_status = 'completed'
        config.save()

        response = self.client.post(
            '/api/import/supplement/',
            {
                'folder_path': str(FIXTURE_PATH),
                'modules': ['hosp', 'icu'],
                'patient_limit': 100,
                'generate_fhir': False,
            },
            format='json',
        )
        self.assertEqual(
            response.status_code, 202,
            f"Supplement import returned {response.status_code}, expected 202. "
            "Check SupplementImportView in core/views.py returns HTTP_202_ACCEPTED."
        )
        self.assertEqual(
            response.json()['status'], 'started',
            "Supplement response should return {status: 'started'}."
        )

        # Wait for background thread to finish
        import time
        for _ in range(30):
            s = self.client.get('/api/import/status/').json()
            if s['import_status'] in ('completed', 'failed'):
                break
            time.sleep(0.5)

        config.refresh_from_db()
        self.assertEqual(
            config.import_status, 'completed',
            f"Supplement import did not complete — status is '{config.import_status}'. "
            f"Progress: {config.import_progress}. The background import thread may "
            "have failed. Check core/importer.py for errors."
        )



from django.urls import path

from . import views

urlpatterns = [
    path('fhir/metadata', views.CapabilityStatementView.as_view(), name='fhir-metadata'),

    path('fhir/Patient/', views.PatientView.as_view(), name='fhir-patient-search'),
    path('fhir/Patient/<str:resource_id>/', views.PatientView.as_view(), name='fhir-patient-read'),
    path('fhir/Patient/<str:resource_id>/$everything', views.PatientEverythingView.as_view(), name='fhir-patient-everything'),

    path('fhir/Encounter/', views.EncounterView.as_view(), name='fhir-encounter-search'),
    path('fhir/Encounter/<str:resource_id>/', views.EncounterView.as_view(), name='fhir-encounter-read'),

    path('fhir/Observation/', views.ObservationView.as_view(), name='fhir-observation-search'),
    path('fhir/Observation/<str:resource_id>/', views.ObservationView.as_view(), name='fhir-observation-read'),

    path('fhir/Condition/', views.ConditionView.as_view(), name='fhir-condition-search'),
    path('fhir/Condition/<str:resource_id>/', views.ConditionView.as_view(), name='fhir-condition-read'),

    path('fhir/Procedure/', views.ProcedureView.as_view(), name='fhir-procedure-search'),
    path('fhir/Procedure/<str:resource_id>/', views.ProcedureView.as_view(), name='fhir-procedure-read'),

    path('fhir/MedicationRequest/', views.MedicationRequestView.as_view(), name='fhir-medicationrequest-search'),
    path('fhir/MedicationRequest/<str:resource_id>/', views.MedicationRequestView.as_view(), name='fhir-medicationrequest-read'),

    path('fhir/DocumentReference/', views.DocumentReferenceView.as_view(), name='fhir-documentreference-search'),
    path('fhir/DocumentReference/<str:resource_id>/', views.DocumentReferenceView.as_view(), name='fhir-documentreference-read'),
]



from django.urls import path
from . import views

urlpatterns = [
    # Cohorts
    path('research/cohorts/', views.CohortListCreateView.as_view(), name='cohort-list-create'),
    path('research/cohorts/compare/', views.CohortCompareView.as_view(), name='cohort-compare'),
    path('research/cohorts/<int:pk>/', views.CohortDetailView.as_view(), name='cohort-detail'),
    path('research/cohorts/<int:pk>/execute/', views.CohortExecuteView.as_view(), name='cohort-execute'),
    path('research/cohorts/<int:pk>/stats/', views.CohortStatsView.as_view(), name='cohort-stats'),
    path('research/cohorts/<int:pk>/members/', views.CohortMembersView.as_view(), name='cohort-members'),

    # Saved Queries
    path('research/queries/', views.QueryListCreateView.as_view(), name='query-list-create'),
    path('research/queries/<int:pk>/', views.QueryDetailView.as_view(), name='query-detail'),
    path('research/queries/<int:pk>/run/', views.QueryRunView.as_view(), name='query-run'),

    # Search & Export
    path('research/search/', views.SearchView.as_view(), name='research-search'),
    path('research/export/', views.ExportView.as_view(), name='research-export'),
]



"""
Custom test runner for mimic-explorer --test.
Shows user-friendly pass/fail output with descriptions.
"""
import logging
import sys
import unittest
import warnings
from django.test.runner import DiscoverRunner


class FriendlyTestResult(unittest.TextTestResult):
    """Test result that shows readable descriptions for each test."""

    def __init__(self, stream, descriptions, verbosity):
        super().__init__(stream, descriptions, verbosity)
        self.successes = []

    def addSuccess(self, test):
        super().addSuccess(test)
        self.successes.append(test)
        doc = test.shortDescription() or str(test)
        self.stream.write(f'  PASS  {doc}\n')
        self.stream.flush()

    def addFailure(self, test, err):
        super().addFailure(test, err)
        doc = test.shortDescription() or str(test)
        self.stream.write(f'  FAIL  {doc}\n')
        # Print the failure message (the helpful msg= text)
        tb_text = self._exc_info_to_string(err, test)
        # Extract just the AssertionError message, not the full traceback
        for line in tb_text.strip().split('\n'):
            if line.startswith('AssertionError') or line.startswith('AssertionError:'):
                self.stream.write(f'        {line}\n')
                break
        else:
            # Fallback: show last line of traceback
            last_line = tb_text.strip().split('\n')[-1]
            self.stream.write(f'        {last_line}\n')
        self.stream.write('\n')
        self.stream.flush()

    def addError(self, test, err):
        super().addError(test, err)
        doc = test.shortDescription() or str(test)
        self.stream.write(f'  ERROR {doc}\n')
        tb_text = self._exc_info_to_string(err, test)
        last_line = tb_text.strip().split('\n')[-1]
        self.stream.write(f'        {last_line}\n\n')
        self.stream.flush()

    def addSkip(self, test, reason):
        super().addSkip(test, reason)
        doc = test.shortDescription() or str(test)
        self.stream.write(f'  SKIP  {doc}\n')
        self.stream.write(f'        Reason: {reason}\n')
        self.stream.flush()

    def printErrors(self):
        # Only print detailed tracebacks for failures/errors
        if self.failures or self.errors:
            self.stream.write('\n' + '=' * 60 + '\n')
            self.stream.write('DETAILED FAILURE/ERROR INFO:\n')
            self.stream.write('=' * 60 + '\n')
            super().printErrors()


class FriendlyTestRunner(unittest.TextTestRunner):
    """Test runner that uses FriendlyTestResult."""
    resultclass = FriendlyTestResult

    def run(self, test):
        self.stream.write('\n')
        self.stream.write('  MIMIC Explorer Test Suite\n')
        self.stream.write('  ' + '=' * 40 + '\n\n')

        # Run the tests — collect results without the default summary
        result = self._makeResult()
        test(result)

        # Print detailed failure tracebacks if any
        result.printErrors()

        # Print our clean summary
        self.stream.write('\n  ' + '-' * 40 + '\n')
        total = result.testsRun
        passed = len(result.successes)
        failed = len(result.failures)
        errors = len(result.errors)
        skipped = len(result.skipped)

        self.stream.write(f'  Results: {passed} passed')
        if failed:
            self.stream.write(f', {failed} failed')
        if errors:
            self.stream.write(f', {errors} errors')
        if skipped:
            self.stream.write(f', {skipped} skipped')
        self.stream.write(f' ({total} total)\n')

        if failed == 0 and errors == 0:
            self.stream.write('  All tests passed! Your installation is working correctly.\n')
        else:
            self.stream.write('\n  Some tests failed. Read the messages above for details\n')
            self.stream.write('  on what went wrong and where to look to fix it.\n')

        self.stream.write('\n')
        return result


class MIMICTestRunner(DiscoverRunner):
    """Django test runner that suppresses migrations noise and uses friendly output."""

    def __init__(self, **kwargs):
        kwargs['verbosity'] = 0  # Suppress migration output
        super().__init__(**kwargs)

    def get_resultclass(self):
        return FriendlyTestResult

    def setup_test_environment(self, **kwargs):
        super().setup_test_environment(**kwargs)
        # Suppress noisy warnings that clutter test output
        warnings.filterwarnings('ignore', message='.*naive datetime.*', category=RuntimeWarning)
        # Suppress SQLite threading noise from background-thread import tests
        logging.getLogger('django.db.backends').setLevel(logging.CRITICAL)
        # Suppress unhandled thread exception tracebacks (e.g. SQLite lock in
        # background import thread tests — the test itself handles the outcome)
        import threading
        threading.excepthook = lambda args: None

    def run_suite(self, suite, **kwargs):
        runner = FriendlyTestRunner(
            stream=sys.stderr,
            verbosity=0,
        )
        return runner.run(suite)



from django.apps import AppConfig
from django.db.backends.signals import connection_created


def set_sqlite_pragmas(sender, connection, **kwargs):
    if connection.vendor == 'sqlite':
        cursor = connection.cursor()
        cursor.execute('PRAGMA journal_mode=WAL;')
        cursor.execute('PRAGMA synchronous=NORMAL;')
        cursor.execute('PRAGMA cache_size=-64000;')
        cursor.execute('PRAGMA foreign_keys=ON;')
        cursor.execute('PRAGMA temp_store=MEMORY;')
        cursor.execute('PRAGMA mmap_size=268435456;')


class CoreConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'core'

    def ready(self):
        connection_created.connect(set_sqlite_pragmas)



"""Research workbench API views (cohorts, saved queries, search, export)."""

import csv
import io
from django.http import StreamingHttpResponse
from rest_framework import status
from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework.response import Response
from rest_framework.views import APIView

from core.duckdb_queries import (
    execute_criteria, get_cohort_patient_data, get_export_data,
    get_patient_ids_for_subject_ids,
)
from core.duckdb_manager import get_connection

def _conn():
    """Return a new cursor for thread-safe concurrent access."""
    return get_connection().cursor()

from .models import SavedQuery, CohortDefinition, CohortMember
from .serializers import (
    SavedQuerySerializer,
    CohortDefinitionSerializer,
    CohortDefinitionListSerializer,
)


# --- Cohort CRUD ---

class CohortListCreateView(ListCreateAPIView):
    queryset = CohortDefinition.objects.all().order_by('-created_at')

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return CohortDefinitionListSerializer
        return CohortDefinitionSerializer


class CohortDetailView(RetrieveUpdateDestroyAPIView):
    queryset = CohortDefinition.objects.all()
    serializer_class = CohortDefinitionSerializer


class CohortExecuteView(APIView):
    """POST /api/research/cohorts/<pk>/execute/ — run criteria engine and populate members."""

    def post(self, request, pk):
        try:
            cohort = CohortDefinition.objects.get(pk=pk)
        except CohortDefinition.DoesNotExist:
            return Response({'error': 'Cohort not found'}, status=status.HTTP_404_NOT_FOUND)

        results = execute_criteria(cohort.criteria)

        # Clear existing members and repopulate
        cohort.members.all().delete()
        members = [
            CohortMember(
                cohort=cohort,
                patient_id=r['patient_id'],
                encounter_id=r['encounter_id'],
                group_label=r.get('group_label', ''),
            )
            for r in results
        ]
        CohortMember.objects.bulk_create(members)

        # Update counts
        cohort.patient_count = len(set(r['patient_id'] for r in results))
        cohort.encounter_count = len(results)
        cohort.save(update_fields=['patient_count', 'encounter_count'])

        # Build response for first 100 members
        member_data = []
        conn = _conn()
        for m in cohort.members.all()[:100]:
            # Resolve patient subject_id from DuckDB
            row = conn.execute(
                "SELECT subject_id FROM patient WHERE id = ?", [m.patient_id]
            ).fetchone()
            enc_row = None
            if m.encounter_id:
                enc_row = conn.execute(
                    "SELECT hadm_id FROM encounter WHERE id = ?", [m.encounter_id]
                ).fetchone()
            member_data.append({
                'id': m.id, 'cohort': m.cohort_id,
                'subject_id': row[0] if row else None,
                'hadm_id': enc_row[0] if enc_row else None,
                'group_label': m.group_label,
            })

        return Response({
            'patient_count': cohort.patient_count,
            'encounter_count': cohort.encounter_count,
            'members': member_data,
        })


def _cohort_stats(cohort):
    """Compute summary statistics for a cohort."""
    patient_ids = list(cohort.members.values_list('patient_id', flat=True).distinct())
    encounter_ids = list(
        cohort.members.exclude(encounter_id__isnull=True)
        .values_list('encounter_id', flat=True)
    )

    data = get_cohort_patient_data(patient_ids, encounter_ids)
    patients = data['patients']
    encounters = data['encounters']

    # Age stats
    ages = [p[3] for p in patients]
    avg_age = round(sum(ages) / len(ages), 1) if ages else None

    # Gender distribution
    gender_dist = {}
    for p in patients:
        g = p[2]
        gender_dist[g] = gender_dist.get(g, 0) + 1

    # Mortality
    mortality = sum(1 for e in encounters if e[5])
    total_enc = len(encounters)

    # Age buckets
    age_buckets = {}
    for label, lo, hi in [('18-30', 18, 30), ('31-45', 31, 45), ('46-60', 46, 60),
                           ('61-75', 61, 75), ('76-90', 76, 90), ('90+', 91, 200)]:
        age_buckets[label] = sum(1 for p in patients if lo <= p[3] <= hi)

    # Group label distribution (from SQLite)
    group_dist = {}
    for gl, cnt in (
        cohort.members.exclude(group_label='')
        .values_list('group_label')
        .annotate(count=__import__('django.db.models', fromlist=['Count']).Count('id'))
        .values_list('group_label', 'count')
    ):
        group_dist[gl] = cnt

    return {
        'name': cohort.name,
        'patient_count': len(patients),
        'encounter_count': total_enc,
        'avg_age': avg_age,
        'gender_distribution': gender_dist,
        'age_distribution': age_buckets,
        'mortality_rate': round(mortality / total_enc * 100, 1) if total_enc else 0,
        'mortality_count': mortality,
        'group_distribution': group_dist,
    }


class CohortStatsView(APIView):
    def get(self, request, pk):
        try:
            cohort = CohortDefinition.objects.get(pk=pk)
        except CohortDefinition.DoesNotExist:
            return Response({'error': 'Cohort not found'}, status=status.HTTP_404_NOT_FOUND)
        return Response(_cohort_stats(cohort))


class CohortMembersView(APIView):
    def get(self, request, pk):
        try:
            cohort = CohortDefinition.objects.get(pk=pk)
        except CohortDefinition.DoesNotExist:
            return Response({'error': 'Cohort not found'}, status=status.HTTP_404_NOT_FOUND)

        page = int(request.query_params.get('page', 1))
        page_size = int(request.query_params.get('page_size', 50))
        search = request.query_params.get('search', '')
        group = request.query_params.get('group', '')

        members = cohort.members.all()
        if group:
            members = members.filter(group_label=group)

        total = members.count()
        start = (page - 1) * page_size
        members_page = list(members.order_by('patient_id')[start:start + page_size])

        # Resolve patient/encounter data from DuckDB
        conn = _conn()
        data = []
        for m in members_page:
            p_row = conn.execute(
                "SELECT subject_id, gender, anchor_age FROM patient WHERE id = ?",
                [m.patient_id]
            ).fetchone()
            if search and p_row and search not in str(p_row[0]):
                continue

            enc_row = None
            if m.encounter_id:
                enc_row = conn.execute(
                    "SELECT hadm_id, admittime, admission_type, hospital_expire_flag FROM encounter WHERE id = ?",
                    [m.encounter_id]
                ).fetchone()

            data.append({
                'subject_id': p_row[0] if p_row else None,
                'gender': p_row[1] if p_row else None,
                'anchor_age': p_row[2] if p_row else None,
                'group_label': m.group_label,
                'hadm_id': enc_row[0] if enc_row else None,
                'admittime': str(enc_row[1]) if enc_row and enc_row[1] else None,
                'admission_type': enc_row[2] if enc_row else None,
                'hospital_expire_flag': bool(enc_row[3]) if enc_row else None,
            })

        return Response({
            'count': total,
            'page': page,
            'page_size': page_size,
            'results': data,
        })


class CohortCompareView(APIView):
    def post(self, request):
        cohort_a_id = request.data.get('cohort_a')
        cohort_b_id = request.data.get('cohort_b')

        if not cohort_a_id or not cohort_b_id:
            return Response({'error': 'cohort_a and cohort_b are required'},
                            status=status.HTTP_400_BAD_REQUEST)

        try:
            cohort_a = CohortDefinition.objects.get(pk=cohort_a_id)
            cohort_b = CohortDefinition.objects.get(pk=cohort_b_id)
        except CohortDefinition.DoesNotExist:
            return Response({'error': 'Cohort not found'}, status=status.HTTP_404_NOT_FOUND)

        return Response({
            'cohort_a': _cohort_stats(cohort_a),
            'cohort_b': _cohort_stats(cohort_b),
        })


# --- Saved Queries ---

class QueryListCreateView(ListCreateAPIView):
    queryset = SavedQuery.objects.all().order_by('-created_at')
    serializer_class = SavedQuerySerializer


class QueryDetailView(RetrieveUpdateDestroyAPIView):
    queryset = SavedQuery.objects.all()
    serializer_class = SavedQuerySerializer


class QueryRunView(APIView):
    def post(self, request, pk):
        try:
            query = SavedQuery.objects.get(pk=pk)
        except SavedQuery.DoesNotExist:
            return Response({'error': 'Query not found'}, status=status.HTTP_404_NOT_FOUND)

        results = execute_criteria(query.query_definition)
        query.result_count = len(results)
        query.save(update_fields=['result_count'])

        conn = _conn()
        encounter_ids = [r['encounter_id'] for r in results if r.get('encounter_id')][:100]

        data = []
        for eid in encounter_ids:
            row = conn.execute("""
                SELECT e.hadm_id, p.subject_id, e.admittime, e.dischtime, e.admission_type
                FROM encounter e JOIN patient p ON e.patient_id = p.id
                WHERE e.id = ?
            """, [eid]).fetchone()
            if row:
                data.append({
                    'hadm_id': row[0], 'subject_id': row[1],
                    'admittime': str(row[2]) if row[2] else None,
                    'dischtime': str(row[3]) if row[3] else None,
                    'admission_type': row[4],
                })

        return Response({
            'total_results': len(results),
            'encounters': data,
        })


# --- Search ---

class SearchView(APIView):
    def post(self, request):
        criteria = request.data.get('criteria', {})
        page = int(request.data.get('page', 1))
        page_size = int(request.data.get('page_size', 50))

        results = execute_criteria(criteria)

        total = len(results)
        start = (page - 1) * page_size
        end = start + page_size
        page_results = results[start:end]

        conn = _conn()
        data = []
        for r in page_results:
            enc_row = conn.execute("""
                SELECT e.hadm_id, p.subject_id, e.admittime, e.dischtime, e.admission_type
                FROM encounter e JOIN patient p ON e.patient_id = p.id
                WHERE e.id = ?
            """, [r['encounter_id']]).fetchone()
            if enc_row:
                dx_rows = conn.execute("""
                    SELECT icd_code, long_title FROM diagnosis
                    WHERE encounter_id = ? ORDER BY seq_num LIMIT 5
                """, [r['encounter_id']]).fetchall()
                data.append({
                    'patient_id': r['patient_id'],
                    'subject_id': enc_row[1],
                    'hadm_id': enc_row[0],
                    'admittime': str(enc_row[2]) if enc_row[2] else None,
                    'dischtime': str(enc_row[3]) if enc_row[3] else None,
                    'admission_type': enc_row[4],
                    'group_label': r.get('group_label', ''),
                    'diagnoses': [{'icd_code': d[0], 'title': d[1]} for d in dx_rows],
                })

        return Response({
            'count': total,
            'page': page,
            'page_size': page_size,
            'results': data,
        })


# --- Export ---

class ExportView(APIView):
    def post(self, request):
        fmt = request.data.get('format', 'csv')
        cohort_id = request.data.get('cohort_id')
        patient_ids = request.data.get('patient_ids', [])
        data_types = request.data.get('data_types', [])

        if not data_types:
            return Response({'error': 'data_types is required'},
                            status=status.HTTP_400_BAD_REQUEST)

        # Resolve patient set
        if cohort_id:
            try:
                cohort = CohortDefinition.objects.get(pk=cohort_id)
            except CohortDefinition.DoesNotExist:
                return Response({'error': 'Cohort not found'}, status=status.HTTP_404_NOT_FOUND)
            patient_pks = list(cohort.members.values_list('patient_id', flat=True).distinct())
        elif patient_ids:
            patient_pks = get_patient_ids_for_subject_ids(patient_ids)
        else:
            return Response({'error': 'cohort_id or patient_ids required'},
                            status=status.HTTP_400_BAD_REQUEST)

        if not patient_pks:
            return Response({'error': 'No patients found'}, status=status.HTTP_404_NOT_FOUND)

        export_data = get_export_data(patient_pks, data_types)

        if fmt == 'json':
            return Response(export_data)

        # CSV streaming response
        def csv_rows():
            for dtype, rows in export_data.items():
                if not rows:
                    continue
                headers = list(rows[0].keys())
                output = io.StringIO()
                writer = csv.writer(output)
                writer.writerow([f'--- {dtype} ---'])
                writer.writerow(headers)
                yield output.getvalue()
                output.seek(0)
                output.truncate(0)

                for row in rows:
                    writer.writerow([str(row.get(h, '')) for h in headers])
                    yield output.getvalue()
                    output.seek(0)
                    output.truncate(0)

                writer.writerow([])
                yield output.getvalue()
                output.seek(0)
                output.truncate(0)

        response = StreamingHttpResponse(csv_rows(), content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="mimic_export.csv"'
        return response
